
classdef MokuLaserLockBox < moku
    % MOKULASERLOCKBOX(IpAddr) 
    % Driver class for the Moku:LaserLockBox
    %
    % MOKULASERLOCKBOX Properties:
    %   IP - IP address of the currently connected Moku:Lab device.
    %   Instrument - Name of the currently running instrument.
    %   Timeout - Command timeout period (sec)
    %
    % MOKULASERLOCKBOX Methods:
    %   commit  - Apply all modified settings.
    %   get_data  - Get full-resolution data from the instrument.
    %   get_frontend  - Get the analog frontend configuration.
    %   get_realtime_data  - Get downsampled data from the instrument with low latency.
    %   get_samplerate  - :return: The current instrument sample rate (Hz) 
    %   set_aux_sine  - Configure the aux sine signal.
    %   set_channel_pid_enables  - Enable or disable connection of the selected PID controller to it's corresponding output channel. Fast = Output 1, Slow = Output 2. 
    %   set_custom_filter  - Configure the filter coefficients in the IIR filter.
    %   set_defaults  - Reset the Oscilloscope to sane defaults. 
    %   set_framerate  - Set framerate 
    %   set_frontend  - Configures gain, coupling and termination for each channel.
    %   set_input_gain  - Set the main input gain (Input Channel 1).
    %   set_local_oscillator  - Configure the local oscillator. The local oscillator can be generated internally with configurable frequency and phase offset,
    %   set_monitor  - Select the point inside the laser lock box to monitor.
    %   set_offsets  - Set offsets throughout the laser locker.
    %   set_output_enables  - Enable or disable the selected output channel.
    %   set_output_range  - Set upper and lower bounds for the signal on each DAC channel. The auxilliary waveform is not restricted to these
    %   set_pid_by_freq  - Configure the selected PID controller using crossover frequencies.
    %   set_pid_by_gain  - Configure the selected PID controller using gain coefficients.
    %   set_pid_enables  - Enable or disable the selected PID controller.
    %   set_precision_mode  - Change aquisition mode between downsampling and decimation.
    %   set_samplerate  - Manually set the sample rate of the instrument.
    %   set_scan  - Configure the scanning generator.
    %   set_timebase  - Set the left- and right-hand span for the time axis.
    %   set_trigger  - Set the trigger source for the monitor channel signals. This can be either of the input or
    %   set_xmode  - Set rendering mode for the horizontal axis.
    %
    % See also moku

    methods
        function obj = MokuLaserLockBox(IpAddr)
            obj@moku(IpAddr, 'laserlockbox');
        end

        function commit(obj)
            % COMMIT()
            % Apply all modified settings.
            % 
            % .. note::
            % 
            %     If the `autocommit` feature has been turned off, this function can be used to manually apply any instrument
            %     settings to the Moku device. These instrument settings are those configured by calling all *set_* and *gen_* type
            %     functions. Manually calling this function allows you to atomically apply many instrument settings at once.
            params = struct();
            mokuctl(obj, 'commit', params);
        end

        function ret = get_data(obj, varargin)
            % GET_DATA()
            % GET_DATA(...,param1,val1,param2,val2,...)
            % Get full-resolution data from the instrument.
            % 
            % This will pause the instrument and download the entire contents of the instrument's
            % internal memory. This may include slightly more data than the instrument is set up
            % to record due to rounding of some parameters in the instrument.
            % 
            % All settings must be committed before you call this function. If *pymoku.autocommit=True*
            % (the default) then this will always be true, otherwise you will need to have called
            % :any:`commit` first.
            % 
            % The download process may take a second or so to complete. If you require high rate
            % data, e.g. for rendering a plot, see `get_realtime_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_data', params);
        end

        function ret = get_frontend(obj, channel)
            % GET_FRONTEND(channel)
            % Get the analog frontend configuration.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel for which the relay settings are being retrieved
            % 
            % :return: Array of bool with the front end configuration of channels
            %         - [0] 50 Ohm
            %         - [1] 10xAttenuation
            %         - [2] AC Coupling
            params = struct('channel',channel);
            ret = ...
            mokuctl(obj, 'get_frontend', params);
        end

        function ret = get_realtime_data(obj, varargin)
            % GET_REALTIME_DATA()
            % GET_REALTIME_DATA(...,param1,val1,param2,val2,...)
            % Get downsampled data from the instrument with low latency.
            % 
            % Returns a new :any:`InstrumentData` subclass (instrument-specific), containing
            % a version of the data that may have been downsampled from the original in order to
            % be transferred quickly.
            % 
            % This function always returns a new object at `framerate` (10Hz by default), whether
            % or not there is new data in that object. This can be verified by checking the return
            % object's *waveformid* parameter, which increments each time a new waveform is captured
            % internally.
            % 
            % The downsampled, low-latency nature of this data makes it particularly suitable for
            % plotting in real time. If you require high-accuracy, high-resolution data for analysis,
            % see `get_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_realtime_data', params);
        end

        function ret = get_samplerate(obj)
            % GET_SAMPLERATE()
            % :return: The current instrument sample rate (Hz) 
            params = struct();
            ret = ...
            mokuctl(obj, 'get_samplerate', params);
        end

        function set_aux_sine(obj, varargin)
            % SET_AUX_SINE()
            % SET_AUX_SINE(...,param1,val1,param2,val2,...)
            % Configure the aux sine signal.
            % 
            % :type amplitude : float; [0, 2.0] Vpp
            % :param amplitude : Auxiliary sine wave amplitude
            % 
            % :type frequency : float; [0, 200e6] Hz
            % :param frequency : Auxiliary sine wave frequency
            % 
            % :type phase : float; [0, 360] degrees
            % :param phase : float; Auxiliary sine wave phase offset
            % 
            % :type sync_to_lo : bool
            % :param sync_to_lo : True = enable phase synchronisation to local oscillator, False = use auxiliary frequency and phase values. 
            % 
            % :type output : list; ['out1', 'out2', 'none']
            % :param output : select which channel to output the auxiliary signal on. 
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_aux_sine', params);
        end

        function set_channel_pid_enables(obj, pid_block, varargin)
            % SET_CHANNEL_PID_ENABLES(pid_block)
            % SET_CHANNEL_PID_ENABLES(...,param1,val1,param2,val2,...)
            % Enable or disable connection of the selected PID controller to it's corresponding output channel. Fast = Output 1, Slow = Output 2. 
            % 
            % :type pid_block : int; [1, 2]
            % :param pid_block : PID controller - 1 = Fast, 2 = Slow 
            % 
            % :type en : bool;
            % :param en : enable or disable channel.
            params = struct('pid_block',pid_block);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_channel_pid_enables', params);
        end

        function set_custom_filter(obj, filt_coeffs)
            % SET_CUSTOM_FILTER(filt_coeffs)
            % Configure the filter coefficients in the IIR filter.
            % 
            % :type filt_coeffs: array;
            % :param filt_coeffs: array containg Direct-Form 1 SOS filter coefficients in the following format:
            % 
            % +-----+------+------+------+------+-------+
            % | s1  | b0.1 | b1.1 | b2.1 | a1.1 |  a2.1 |
            % +-----+------+------+------+------+-------+
            % | s2  | b0.2 | b1.2 | b2.2 | a1.2 |  a2.2 |
            % +-----+------+------+------+------+-------+
            % 
            % Each 'a' coefficient must be a float in the range [-2.0, +2.0). 's' coefficients are multiplied into each 'b' coefficient before being sent to the device. 
            % These products (sN x b0.N, sN x b1.N, sN x b2.N) must also fall in the range [-2.0, +2.0). Internally, the 'a' and 'b' coefficients are represented as 
            % signed 32-bit fixed-point numbers, with 30 fractional bits. 
            % 
            % Filter coefficients can be computed using signal processing toolboxes in e.g. MATLAB or SciPy.
            params = struct('filt_coeffs',filt_coeffs);
            mokuctl(obj, 'set_custom_filter', params);
        end

        function set_defaults(obj)
            % SET_DEFAULTS()
            % Reset the Oscilloscope to sane defaults. 
            params = struct();
            mokuctl(obj, 'set_defaults', params);
        end

        function set_framerate(obj, fr)
            % SET_FRAMERATE(fr)
            % Set framerate 
            params = struct('fr',fr);
            mokuctl(obj, 'set_framerate', params);
        end

        function ret = set_frontend(obj, channel, varargin)
            % SET_FRONTEND(channel)
            % SET_FRONTEND(...,param1,val1,param2,val2,...)
            % Configures gain, coupling and termination for each channel.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel to which the settings should be applied
            % 
            % :type fiftyr: bool
            % :param fiftyr: 50Ohm termination; default is 1MOhm.
            % 
            % :type atten: bool
            % :param atten: Turn on 10x attenuation. Changes the dynamic range between 1Vpp and 10Vpp.
            % 
            % :type ac: bool
            % :param ac: AC-couple; default DC.
            params = struct('channel',channel);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'set_frontend', params);
        end

        function set_input_gain(obj, varargin)
            % SET_INPUT_GAIN()
            % SET_INPUT_GAIN(...,param1,val1,param2,val2,...)
            % Set the main input gain (Input Channel 1).
            % 
            % :type gain: int; {-20, 0, 24, 48} dB
            % :param gain: Input gain
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_input_gain', params);
        end

        function set_local_oscillator(obj, varargin)
            % SET_LOCAL_OSCILLATOR()
            % SET_LOCAL_OSCILLATOR(...,param1,val1,param2,val2,...)
            % Configure the local oscillator. The local oscillator can be generated internally with configurable frequency and phase offset,
            % externally via ADC 2 or internally with a PLL phase locked to ADC 2 plus an additional phase offset.
            % 
            % :type frequency : float; [0, 200e6] Hz
            % :param frequency : Internal demodulation frequency
            % 
            % :type phase : float; [0, 360] degrees
            % :param phase : float; Internal demodulation phase
            % 
            % :type source : list; ['internal', 'external', 'external_pll']
            % :param source : Local Oscillator Source
            % 
            % :type pll_auto_acq : bool
            % :param pll_auto_acq : Enable PLL Auto Acquire
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_local_oscillator', params);
        end

        function set_monitor(obj, monitor_ch, source)
            % SET_MONITOR(monitor_ch, source)
            % Select the point inside the laser lock box to monitor.
            % 
            % There are two monitoring channels available, 'A' and 'B'; you can mux any of the internal
            % monitoring points to either of these channels.
            % 
            % The source is one of:
            %         - **error_signal**: error signal (after low-pass filter)
            %         - **pid_fast**: output of the fast pid
            %         - **pid_slow**: output of the slow pid
            %         - **offset_fast**: offset on the input to the fast pid
            %         - **in1**: input channel 1
            %         - **in2**: input channel 2
            %         - **out1**: output channel 1
            %         - **out2**: output channel 2
            %         - **scan**: scan signal
            %         - **lo**: local oscillator signal
            %         - **aux**: auxiliary sinewave signal
            % 
            % :type monitor_ch: string; {'A','B'}
            % :param monitor_ch: Monitor channel
            % :type source: string; {'error', 'pid_fast', 'pid_slow', 'offset_fast', 'offset_slow', 'in1', 'in2', 'out1', 'out2', 'scan', 'lo', 'aux', 'slow_scan'}
            % :param source: Signal to monitor
            params = struct('monitor_ch',monitor_ch,'source',source);
            mokuctl(obj, 'set_monitor', params);
        end

        function set_offsets(obj, position, offset)
            % SET_OFFSETS(position, offset)
            % Set offsets throughout the laser locker.
            % 
            % :type position : list; ['pid_input', 'out1', 'out2']
            % :param position : The desired point to add an offset
            % 
            % :type offset : float, [-2.0, 2.0] Volts.
            % :param offset : voltage offset. 
            params = struct('position',position,'offset',offset);
            mokuctl(obj, 'set_offsets', params);
        end

        function set_output_enables(obj, ch, varargin)
            % SET_OUTPUT_ENABLES(ch)
            % SET_OUTPUT_ENABLES(...,param1,val1,param2,val2,...)
            % Enable or disable the selected output channel.
            % 
            % :type ch : int; [1, 2]
            % :param ch : 1 = Output 1, 2 = Output 2 
            % 
            % :type en : bool;
            % :param en : enable or disable channel.
            params = struct('ch',ch);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_output_enables', params);
        end

        function set_output_range(obj, ch, maximum, minimum)
            % SET_OUTPUT_RANGE(ch, maximum, minimum)
            % Set upper and lower bounds for the signal on each DAC channel. The auxilliary waveform is not restricted to these
            % bounds when added to either DAC channel.  
            % 
            % :type ch : int; [1, 2]
            % :param ch : 1 = Output 1, 2 = Output 2 
            % 
            % :type maximum: float, [-1.0, 1.0] Volts;
            % :param maximum: maximum value the output signal can be before clipping occurs.
            % 
            % :type minimum: float, [-1.0, 1.0] Volts;
            % :param maximum: maximum value the output signal can be before clipping occurs.
            params = struct('ch',ch,'maximum',maximum,'minimum',minimum);
            mokuctl(obj, 'set_output_range', params);
        end

        function set_pid_by_freq(obj, pid_block, varargin)
            % SET_PID_BY_FREQ(pid_block)
            % SET_PID_BY_FREQ(...,param1,val1,param2,val2,...)
            % Configure the selected PID controller using crossover frequencies.
            % 
            % :type pid_block : int; [1,2]
            % :param pid_block : PID controller - 1 = Fast, 2 = Slow 
            % 
            % :type kp: float; [-1e3,1e3]
            % :param kp: Proportional gain factor
            % 
            % :type i_xover: float; [1e-3,1e6] Hz
            % :param i_xover: Integrator crossover frequency
            % 
            % :type d_xover: float; [1,10e6] Hz
            % :param d_xover: Differentiator crossover frequency
            % 
            % :type si: float; float; [-1e3,1e3]
            % :param si: Integrator gain saturation
            % 
            % :type sd: float; [-1e3,1e3]
            % :param sd: Differentiator gain saturation
            % 
            % :type enable: bool;
            % :param enable: enables pid outputs
            % 
            % :raises InvalidConfigurationException: if the configuration of PID gains is not possible.
            params = struct('pid_block',pid_block);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_pid_by_freq', params);
        end

        function set_pid_by_gain(obj, pid_block, varargin)
            % SET_PID_BY_GAIN(pid_block)
            % SET_PID_BY_GAIN(...,param1,val1,param2,val2,...)
            % Configure the selected PID controller using gain coefficients.
            % 
            % :type pid_block : int; [1,2]
            % :param pid_block : PID controller - 1 = Fast, 2 = Slow 
            % 
            % :type g: float; [0,2^16 - 1]
            % :param g: Gain
            % 
            % :type kp: float; [-1e3,1e3]
            % :param kp: Proportional gain factor
            % 
            % :type ki: float; 
            %         [0, 31.25e6] with a resolution of 31.25e6 / 2^24-1 when pid_block = 1
            %         [0, 488.28e3] with a resolution of 488.28 / 2^24-1 when pid_block = 2.
            % :param ki: Integrator gain factor. 
            % 
            % :type kd: float; [0, 31.25e6] when pid_block = 1, [0, 488.28e3] when pid_block = 2
            % :param kd: Differentiator gain factor
            % 
            % :type si: float; float; [-1e3,1e3]
            % :param si: Integrator gain saturation
            % 
            % :type sd: float; [-1e3,1e3]
            % :param sd: Differentiator gain saturation
            % 
            % :type enable: bool;
            % :param enable: enables pid outputs
            % 
            % :raises InvalidConfigurationException: if the configuration of PID gains is not possible.
            params = struct('pid_block',pid_block);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_pid_by_gain', params);
        end

        function set_pid_enables(obj, pid_block, varargin)
            % SET_PID_ENABLES(pid_block)
            % SET_PID_ENABLES(...,param1,val1,param2,val2,...)
            % Enable or disable the selected PID controller.
            % 
            % :type pid_block : int; [1, 2]
            % :param pid_block : PID controller - 1 = Fast, 2 = Slow 
            % 
            % :type en : bool;
            % :param en : enable or disable PID controller described in pid_block.
            params = struct('pid_block',pid_block);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_pid_enables', params);
        end

        function set_precision_mode(obj, state)
            % SET_PRECISION_MODE(state)
            % Change aquisition mode between downsampling and decimation.
            % Precision mode, a.k.a Decimation, samples at full rate and applies a low-pass filter to the data. This improves
            % precision. Normal mode works by direct downsampling, throwing away points it doesn't need.
            % 
            % :param state: Select Precision Mode
            % :type state: bool
            params = struct('state',state);
            mokuctl(obj, 'set_precision_mode', params);
        end

        function set_samplerate(obj, samplerate, varargin)
            % SET_SAMPLERATE(samplerate)
            % SET_SAMPLERATE(...,param1,val1,param2,val2,...)
            % Manually set the sample rate of the instrument.
            % 
            % The sample rate is automatically calculated and set in :any:`set_timebase`.
            % 
            % This interface allows you to specify the rate at which data is sampled, and set
            % a trigger offset in number of samples. This interface is useful for datalogging and capturing
            % of data frames.
            % 
            % :type samplerate: float; *0 < samplerate <= MAX_SAMPLERATE smp/s*
            % :param samplerate: Target samples per second. Will get rounded to the nearest allowable unit.
            % 
            % :type trigger_offset: int; *-2^16 < trigger_offset < 2^31*
            % :param trigger_offset: Number of samples before (-) or after (+) the trigger point to start capturing.
            % 
            % :raises ValueOutOfRangeException: if either parameter is out of range.
            params = struct('samplerate',samplerate);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_samplerate', params);
        end

        function set_scan(obj, varargin)
            % SET_SCAN()
            % SET_SCAN(...,param1,val1,param2,val2,...)
            % Configure the scanning generator.
            % 
            % :type amplitude : float; [0, 2.0] Vpp
            % :param amplitude : Scan amplitude
            % 
            % :type frequency : float; [0, 1e6] Hz
            % :param frequency : Scan frequency
            % 
            % :type phase : float; [0, 360] degrees
            % :param phase : float; Scan phase offset
            % 
            % :type waveform : List; ['sawtooth', 'triangle']
            % :param sync_to_lo : Scan waveform type. 
            % 
            % :type output : list; ['out1', 'out2', 'none']
            % :param output : select which channel to output the scan signal on. 
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_scan', params);
        end

        function set_timebase(obj, t1, t2)
            % SET_TIMEBASE(t1, t2)
            % Set the left- and right-hand span for the time axis.
            % Units are seconds relative to the trigger point.
            % 
            % :type t1: float
            % :param t1:
            %         Time, in seconds, from the trigger point to the left of screen. This may be negative (trigger on-screen)
            %         or positive (trigger off the left of screen).
            % 
            % :type t2: float
            % :param t2: As *t1* but to the right of screen.
            % 
            % :raises InvalidConfigurationException: if the timebase is backwards or zero.
            params = struct('t1',t1,'t2',t2);
            mokuctl(obj, 'set_timebase', params);
        end

        function set_trigger(obj, source, edge, level, varargin)
            % SET_TRIGGER(source, edge, level)
            % SET_TRIGGER(...,param1,val1,param2,val2,...)
            % Set the trigger source for the monitor channel signals. This can be either of the input or
            % monitor signals, the external input or the scan output. 
            % 
            % :type source: string, {'in1','in2','scan','A','B','ext'}
            % :param source: Trigger Source. May be either an input or monitor channel (as set by 
            %                 :py:meth:`~pymoku.instruments.LockInAmp.set_monitor`), external or the scan output. External refers 
            %                 to the back-panel connector of the same name, allowing triggering from an 
            %                 externally-generated digital [LV]TTL or CMOS signal.
            % 
            % :type edge: string, {'rising','falling','both'}
            % :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising)
            %                 or negative (falling), with the 'both' option being invalid.
            % 
            % :type level: float, [-10.0, 10.0] volts
            % :param level: Trigger level
            % 
            % :type minwidth: float, seconds
            % :param minwidth: Minimum Pulse Width. 0 <= minwidth < (2^32/samplerate). Can't be used with maxwidth.
            % 
            % :type maxwidth: float, seconds
            % :param maxwidth: Maximum Pulse Width. 0 <= maxwidth < (2^32/samplerate). Can't be used with minwidth.
            % 
            % :type hysteresis: float, [100e-6, 1.0] volts
            % :param hysteresis: Hysteresis around trigger point.
            % 
            % :type hf_reject: bool
            % :param hf_reject: Enable high-frequency noise rejection
            % 
            % :type mode: string, {'auto', 'normal'}
            % :param mode: Trigger mode.
            % 
            % :type trig_on_scan_rising: bool
            % :param trig_on_scan_rising: trigger only during rising portion of scan signal.
            params = struct('source',source,'edge',edge,'level',level);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_trigger', params);
        end

        function set_xmode(obj, xmode)
            % SET_XMODE(xmode)
            % Set rendering mode for the horizontal axis.
            % 
            % :type xmode: string, {'roll','sweep','fullframe'}
            % :param xmode:
            %         Respectively; Roll Mode (scrolling), Sweep Mode (normal oscilloscope trace sweeping across the screen)
            %         or Full Frame (like sweep, but waits for the frame to be completed).
            params = struct('xmode',xmode);
            mokuctl(obj, 'set_xmode', params);
        end

    end
end
